/*
 *  options.c - source code for all things related to program options
 *
 *	Project 1
 *	CSE 326, Winter 2008
 *	University of Washington
 *
 *  Created by Ilya Maykov on 1/6/05.
 *  Modified by Gyanit Singh.
 *  Copyright 2005 University of Washington. All rights reserved.
 *
 */

#include <stdio.h>
#include <assert.h>
#include <limits.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "options.h"

/*
 *	Internal function to verify that all required options are present and valid.
 */
static int
VerifyOptions(const Options *pOpts);


int
ParseOptions(int argc, const char **argv, Options *pOpts) {
    int i, len;
    assert(argv);
    assert(pOpts);
    memset(pOpts, 0, sizeof(*pOpts));
    pOpts->sortType = kInvalidSort;
    pOpts->sortOrder = kInvalidOrder;
    pOpts->arraySize = 0;
    for(i = 1; i < argc; ++i) {
        if(	strncmp("-h", argv[i], strlen("-h")) == 0 ||
                strncmp("--help", argv[i], strlen("--help")) == 0) {
            PrintUsage(stdout);
            exit(0);
        } else if(strncmp("--insertionsort", argv[i], strlen("--insertionsort")) == 0) {
            pOpts->sortType = kInsertionSort;
        } else if(strncmp("--quicksort1", argv[i], strlen("--quicksort1")) == 0) {
            pOpts->sortType = kQuickSortOne;
        } else if(strncmp("--quicksort2", argv[i], strlen("--quicksort2")) == 0) {
            pOpts->sortType = kQuickSortTwo;
        } else if(strncmp("--imergesort", argv[i], strlen("--imergesort")) == 0) {
            pOpts->sortType = kIterativeMergeSort;
        } else if(strncmp("--rmergesort", argv[i], strlen("--rmergesort")) == 0) {
            pOpts->sortType = kRecursiveMergeSort;
        } else if(strncmp("--bubblesort", argv[i], strlen("--bubblesort")) == 0) {
            pOpts->sortType = kBubbleSort;
        } else if(strncmp("--size=", argv[i], len = strlen("--size=")) == 0) {
            //data set size parsed here
            char* end;
            pOpts->arraySize = strtol(argv[i] + len, &end, 0);
            if( (pOpts->arraySize == LONG_MIN || pOpts->arraySize == LONG_MAX) && errno == ERANGE) {
                fprintf(stderr, "Error: over/underflow parsing \"%s\", setting size to 0\n", argv[i]);
                pOpts->arraySize = 0;
            } else if(*end != '\0') {
                fprintf(stderr, "Warning: strtol() returned before parsing all of \"%s\"\n", argv[i]);
            }
        } else if(strncmp("--input=", argv[i], len = strlen("--input=")) == 0) {
            //order parsed here
            if(strncmp("random", argv[i] + len, strlen("random")) == 0) {
                pOpts->sortOrder = kRandomOrder;
            } else if(strncmp("increasing", argv[i] + len, strlen("increasing")) == 0) {
                pOpts->sortOrder = kIncreasingOrder;
            } else if(strncmp("decreasing", argv[i] + len, strlen("decreasing")) == 0) {
                pOpts->sortOrder = kDecreasingOrder;
            } else if(strncmp("flat", argv[i] + len, strlen("flat")) == 0) {
                pOpts->sortOrder = kFlatOrder;
            } else {
                fprintf(stderr, "Error: bad parameter \"%s\"\n", argv[i]);
            }
        } else {
            fprintf(stderr, "Error: bad parameter \"%s\"\n", argv[i]);
        }
    } //end for()
    return VerifyOptions(pOpts);
}


static int
VerifyOptions(const Options *pOpts) {
    int err = 0;
    assert(pOpts);
    if((unsigned int)pOpts->sortType >= kInvalidSort) {
        fprintf(stderr, "Error: missing or invalid sort type\n");
        err = -1;
    }
    if((unsigned int)pOpts->sortOrder >= kInvalidOrder) {
        fprintf(stderr, "Error: missing or invalid sort order\n");
        err = -1;
    }
    if(pOpts->arraySize < 1) {
        fprintf(stderr, "Error: %d is an invalid array size\n", pOpts->arraySize);
        err = -1;
    }
    return err;
}


int
PrintOptions(const Options *pOpts, FILE *stream) {
    assert(pOpts != NULL);
    assert(stream != NULL);
    assert((unsigned int)pOpts->sortType < kInvalidSort);
    assert((unsigned int)pOpts->sortOrder < kInvalidOrder);
    assert(pOpts->arraySize > 0);
    fprintf(stream, "Selected options:\n\t"
            "Sort type: %s\n\t"
            "Sort order: %s\n\t"
            "Data set size: %d\n",
            kSortTypeStrings[pOpts->sortType],
            kSortOrderStrings[pOpts->sortOrder],
            pOpts->arraySize);
    return 0;
}


int
PrintUsage(FILE *stream) {
    assert(stream);
    fprintf(stream, "Usage: project1 SORTTYPE DATASIZE ORDER\n"
            "Generates an array of DATASIZE integers, initially ordered by ORDER. Sorts the array using SORTTYPE.\n"
            "Sort types:\n\t"
            "--insertionsort		Insertion sort\n\t"
            "--quicksort1		Quicksort Variety 1 (Insertion sort at CUTOFF)\n\t"
            "--quicksort2		Quicksort Variety 2 (return at CUTOFF, single insertion sort at end\n\t"
            "--imergesort		Iterative mergesort\n\t"
            "--rmergesort		Recursive mergesort\n\t"
            "--bubblesort		Bubble sort\n\n"
            "Other options:\n\t"
            "--size=n		Sort an array of n integers\n\t"
            "--input=random		Generate an array of random integers to use as input\n\t"
            "--input=increasing	Generate a sorted array of increasing integers to use as input\n\t"
            "--input=decreasing	Generate a sorted array of decreasing integers to use as input\n\t"
            "--input=flat		Generate an array containing copies of the same integer to use as input\n\t"
            "--help, -h		Print this help\n\n");
    return 0;
}
